ERROR_TEMPLATE = "⚠ [bold red]{}[/bold red]"
COMMAND_UNDONE = "✋ [bold Yellow]Command will not take effect.[/bold Yellow]"
COMMAND_HINT = "👉 [bold Green]You may need to run:[/bold Green] [italic]{}[/italic]"
COMMAND_SUCCESSFUL = "[green]✔️ {}[green]"
OATH = "\n" \
       "    Night gathers, and now my watch begins. \n" \
       "    It shall not end until my death. \n" \
       "    I shall take no wife, hold no lands, father no children. \n" \
       "    I shall wear no crowns and win no glory. \n" \
       "    I shall live and die at my post.\n" \
       "    I am the sword in the darkness. \n" \
       "    I am the watcher on the walls. \n" \
       "    I am the fire that burns against the cold,\n\r" \
       "        the light that brings the dawn,\n\r" \
       "        the horn that wakes the sleepers,\n\r" \
       "        the shield that guards the realms of men. \n\r" \
       "    I pledge my life and honor to the Night’s Watch,\n\r" \
       "        for this night and all the nights to come. \n\r" \
       "    G.R.R. Martin\n\r"
